globalThis.navigator = globalThis.navigator || {};
import Context from './contexts/context.mjs';

function cloneRegExp(re) {
    const regexMatch = /^\/(.*)\/([gimyu]*)$/.exec(re.toString());
    return new RegExp(regexMatch[1], regexMatch[2]);
}
function clone(arg) {
    if (typeof arg !== 'object') {
        return arg;
    }
    if (arg === null) {
        return null;
    }
    if (Array.isArray(arg)) {
        return arg.map(clone);
    }
    if (arg instanceof Date) {
        return new Date(arg.getTime());
    }
    if (arg instanceof RegExp) {
        return cloneRegExp(arg);
    }
    const cloned = {};
    for (const name in arg) {
        if (Object.prototype.hasOwnProperty.call(arg, name)) {
            cloned[name] = clone(arg[name]);
        }
    }
    return cloned;
}

class DiffContext extends Context {
    constructor(left, right) {
        super();
        this.left = left;
        this.right = right;
        this.pipe = 'diff';
    }
    setResult(result) {
        if (this.options.cloneDiffValues && typeof result === 'object') {
            const clone$1 = typeof this.options.cloneDiffValues === 'function'
                ? this.options.cloneDiffValues
                : clone;
            if (typeof result[0] === 'object') {
                result[0] = clone$1(result[0]);
            }
            if (typeof result[1] === 'object') {
                result[1] = clone$1(result[1]);
            }
        }
        return super.setResult(result);
    }
}

export { DiffContext as D, clone as c };
